@@ -125,18 +125,18 @@ test_that("validate_robust2sls() works correctly", {
   t$cons$convergence$criterion <- NULL
   z$cons$convergence$new <- 1:5
   expect_error(validate_robust2sls(t),
-               "Component \\$cons\\$convergence must be a list with 4 elements")
+               "Component \\$cons\\$convergence must be a list with 5 elements")
   expect_error(validate_robust2sls(z),
-               "Component \\$cons\\$convergence must be a list with 4 elements")
+               "Component \\$cons\\$convergence must be a list with 5 elements")
 
   t <- z <- test1
   t$cons$convergence[["diff"]] <- "abc"
   t$cons$convergence$difference <- NULL
   names(z$cons$convergence) <- NULL
   expect_error(validate_robust2sls(t),
-               "Component \\$cons\\$convergence must have 4 named components:")
+               "Component \\$cons\\$convergence must have 5 named components:")
   expect_error(validate_robust2sls(z),
-               "Component \\$cons\\$convergence must have 4 named components:")
+               "Component \\$cons\\$convergence must have 5 named components:")
 
   t <- test1
   t$cons$iterations <- c("abc", "def", "ghi")
@@ -352,7 +352,7 @@ test_that("validate_robust2sls() works correctly", {
   expect_error(validate_robust2sls(z),
             "\\$cons\\$convergence\\$difference must be a numeric value >= 0")
   z <- test1
-  con_list <- list(criterion = 2, difference = NULL, converged = NULL, iter = 1)
+  con_list <- list(criterion = 2, difference = NULL, converged = NULL, iter = 1, max_iter = NULL)
   z$cons$iterations$setting <- "convergence"
   z$cons$convergence <- con_list
   expect_error(validate_robust2sls(z),
@@ -366,7 +366,7 @@ test_that("validate_robust2sls() works correctly", {
   expect_error(validate_robust2sls(z),
       "\\$cons\\$convergence\\$converged must be a logical vector of length 1")
   z <- test1
-  list_conv <- list(criterion = 2, difference = 1, converged = NULL, iter = 1)
+  list_conv <- list(criterion = 2, difference = 1, converged = NULL, iter = 1, max_iter = 10)
   z$cons$iterations$setting <- "convergence"
   z$cons$convergence <- list_conv
   expect_error(validate_robust2sls(z),
@@ -483,6 +483,16 @@ test_that("validate_robust2sls() works correctly", {
   expect_error(validate_robust2sls(t),
                "Element m0 of list \\$type must be a numeric vector that only contains the values -1, 0, or 1")
 
+  # new tests for max_iter argument (test3 used convergence)
+  t <- test3
+  t$cons$convergence$max_iter <- "5"
+  expect_error(validate_robust2sls(t),
+               "x\\$cons\\$convergence\\$max_iter must either be NULL or numeric")
+  z <- test1 # test1 sets numeric iteration
+  z$cons$convergence$max_iter <- 3
+  expect_error(validate_robust2sls(z),
+               "When iterations is numeric, then max_iter must be NULL")
+
   skip_on_cran()
   expect_snapshot_output(test1)
   expect_snapshot_output(test2)
